angular
  .module('cockpit.services')
  //.factory('UserData', function ($localstorage, $state, $ionicPopup, $cordovaGoogleAnalytics, $cordovaLocalNotification) {
  .factory('CockpitData', function($q, UserData) {
    var data = {}

    var getData = function(url, storageKey, cache) {
      return $q(function(resolve, reject) {
        // if (storageKey !== undefined && data[storageKey] !== undefined) {
        //   console.log(data);
        //   if (cache) {
        //     resolve(data[storageKey])
        //     return
        //   }
        // }
        UserData.get(url, storageKey)
          .then(function(result) {
            // data[storageKey] = result
            resolve(result)
          })
          .catch(function(error) {
            reject(error)
          })
      })
    }

    var that = {
      getUser: function() {
        return getData('api', 'user', true)
          .then(function(result) {
            if (result.code != 'OK') {
              return null
            }

            var position = result.data.position
            position.role =
              position.positionCode === 4
                ? 'BR'
                : position.positionCode == 3 ? 'BL' : 'AG'
            position.staffCode =
              position.role === 'AG' ? position.agencyId : position.positionName

            return {
              position: position,
              subs: result.data.subs
            }
          })
          .catch(function() {
            return null
          })
      },

      getLeads: function(limit) {
        return getData('lead', 'leads', true).then(function(result) {
          if (result.code !== 'OK') { return []}
          
          var leads = result.data

          return leads
        })
      },

      getInsurances: function(staff, event, searchTerm) {
        return getData('insurance?employeeId=' + staff + '&eventType=' + event + '&searchTerm=' + searchTerm, staff + event + searchTerm, false).then(function(result) {
          if (result.code !== 'OK') return result
          var insurances = result.data
          return insurances
        })
      },

      getScorings: function(employeeId, dateFrom, dateTo) {
        return getData('scoring?employeeId=' + employeeId + '&dateFrom=' + dateFrom + '&dateTo=' + dateTo, employeeId + dateFrom + dateTo + new Date().toDateString(), false).then(function(result) {
          if (result.code !== 'OK') return result
          var scorings = result.data
          return scorings
        })
      },
     
      getHistory: function(processId) {
        return getData('lead/' + processId + '/history', processId).then(function(result) {
          if (result.code !== 'OK') return null
          var histories = result.data
          return histories
        })
      },
      getFaults: function() {
        return getData('fault', 'faults', true).then(function(result) {
          if (result.code !== 'OK') return null
          var faults = result.data.sort(function (a, b) {
            if (b.Process.IsClosed && b.Process.LeadStatusCode1 !== 'FT-SOL') return 1
            if (a.Process.IsClosed && a.Process.LeadStatusCode1 !== 'FT-SOL') return -1
            if (a.Activity.AddData.Fault.Urgency === '0') return -1
            if (b.Activity.AddData.Fault.Urgency === '0') return 1
            return b.Activity.AddData.Fault.Urgency - a.Activity.AddData.Fault.Urgency
          })
          return faults
        })
      },
      getLeadPhoneNumber: function() {
        return UserData.getPhone()
      },
      getLeadClosedStatuses: function() {
        return getData('lead/closed-statuses').then(function(result) {
          if (result.code !== 'OK') return null
          return result.data.map(function(state) {
            return {
              value: state.LeadStatusId,
              text1: state.LeadStatusName1,
              text2: state.LeadStatusName2
            }
          })
        })
      },
      getLeadSubs: function() {
        return getData('staff/all').then(function(result) {
          if (result.code !== 'OK') return null
          return result.data
        })
      },
      getLeadBrms: function() {
        return getData('staff').then(function(result) {
          if (result.code !== 'OK') return null
          return result.data
        })
      },
      leadClose: function(processId, statusId, note) {
        return $q(function(resolve, reject) {
          UserData.post(
            'lead/' +
              processId +
              '/close?statusId=' +
              statusId +
              '&note=' +
              encodeURIComponent(note),
            {}
          )
            .then(function(result) {
              if (result.code === 'OK') resolve()
              else reject()
            })
            .catch(function() {
              console.log('error closing lead');
              reject()
            })
        })
      },
      leadSchedule: function(processId, assignedTo, date, note) {
        return $q(function(resolve, reject) {
          UserData.post(
            'lead/' +
              processId +
              '/schedule?targetStaff=' +
              encodeURIComponent(assignedTo) +
              '&scheduleDate=' +
              date.toUTCString() +
              '&note=' +
              encodeURIComponent(note),
            {}
          )
            .then(function(result) {
              if (result.code === 'OK' && result.data) resolve()
              else reject()
            })
            .catch(function() {
              reject()
            })
        })
      },
      leadTransfer: function(processId, assignedTo, note) {
        return $q(function(resolve, reject) {
          UserData.post(
            'lead/' +
              processId +
              '/transfer?targetStaff=' +
              encodeURIComponent(assignedTo) +
              '&note=' +
              encodeURIComponent(note),
            {}
          )
            .then(function(result) {
              if (result.code === 'OK') resolve()
              else reject()
            })
            .catch(function() {
              reject()
            })
        })
      },
      leadPayOut: function(processId, productId, loanId, issueValue) {
        return $q(function(resolve, reject) {
          UserData.post(
            'lead/' +
              processId +
              '/payout?note=&productId=' +
              productId +
              '&loanId=' +
              '-1' +
              '&issueValue=' +
              issueValue
          )
            .then(function(result) {
              if (result.code === 'OK') resolve()
              else reject()
            })
            .catch(function() {
              reject()
            })
        })
      },
      leadCall: function(processId) {
        return $q(function(resolve, reject) {
          UserData.post('lead/' + processId + '/call')
            .then(function(result) {
              if (result.code === 'OK') resolve()
              else reject()
            })
            .catch(function() {
              reject()
            })
        })
      },

      getReports: function() {
        return getData('report', 'reports').then(function(result) {
          var reports = result.data.plan;
          return {
            plans: [
              { paramName: 'NC', value: reports.nc.Value },
              { paramName: 'OBN', value: reports.obn.Value },
              { paramName: 'SUB', value: reports.sub.Value },
              { paramName: 'REF', value: reports.ref.Value },
              { paramName: 'PU', value: reports.pu.Value },
              { paramName: 'Prodeje', value: reports.sales.Value },
              { paramName: 'Výběry', value: reports.collections.Value },
              { paramName: 'VZ', value: reports.vz.Value },
              {
                paramName: '5+',
                value: reports.c5Plus.Value + '/' + reports.c5PlusTotal.Value
              },
              {
                paramName: '5=',
                value: reports.c5.Value + '/' + reports.c5Total.Value
              },
              {
                paramName: '8=',
                value: reports.c8.Value + '/' + reports.c8Total.Value
              },
              {
                paramName: '12+',
                value: reports.c12Plus.Value + '/' + reports.c12PlusTotal.Value
              },
              {
                paramName: 'RIA',
                value: reports.ria.Value + '/' + reports.riaTotal.Value
              },
              {
                paramName: 'NRIA',
                value: reports.nria.Value + '/' + reports.nriaTotal.Value
              },
              {
                paramName: 'QC',
                value: reports.qc.Value + '/' + reports.qcTotal.Value
              },
              {
                paramName: 'BTQ',
                value: reports.btq.Value + '/' + reports.btqTotal.Value
              },
              {
                paramName: '12',
                value: reports.c12St.Value + '/' + reports.c12StTotal.Value
              },
              {
                paramName: 'Predikce',
                value:
                  'NC - ' +
                  reports.ncPred.Value +
                  ', Prodeje - ' +
                  reports.salPred.Value +
                  ', Výběry - ' +
                  reports.collPred.Value
              }
            ],
            fulfillment: result.data.fulFillment
          }
        })
      },

      sendReports: function() {
        return $q(function(resolve, reject) {
          UserData.post('report', {})
            .then(function(result) {
              if (result.data.result == 0) {
                resolve()
              } else {
                reject(result.data.errorMessage)
              }
            })
            .catch(function() {
              reject()
            })
        })
      },

      getParamReports: function(paramName) {
        if (paramName == 'Výběry') paramName = 'coll'
        return getData('report/' + paramName.replace('+', '_')).then(function(
          result
        ) {
          if (result.code !== 'OK') return []

          if (
            paramName == '5+' ||
            paramName == '5=' ||
            paramName == '8=' ||
            paramName == '12+' ||
            paramName == '12' ||
            paramName == 'RIA' ||
            paramName == 'NRIA' ||
            paramName == 'QC' ||
            paramName == 'BTQ'
          ) {
            return result.data.map(function(e) {
              return {
                name: e.name,
                value: e.value + '/' + e.valueTotal
              }
            })
          } else if (paramName == 'Predikce') {
            return result.data.map(function(e) {
              return {
                name: e.name,
                value:
                  'NC - ' +
                  e.ncValue +
                  ', Prodeje - ' +
                  e.salValue +
                  ', Výběry - ' +
                  e.collValue
              }
            })
          }
          return result.data
        })
      },

      getStats: function() {
        return getData('target', 'stats', true)
          .then(function(result) {
            if (result.code != 'OK') {
              return { plans: [], month: null }
            }

            var coll = result.data.filter(function(row) {
              return (
                row.desc != 'C1' &&
                row.desc != 'C2' &&
                row.desc != 'L1' &&
                row.desc != 'L2'
              )
            })

            return {
              plans: coll.map(function(e) {
                switch (e.desc) {
                  case 'Coll':
                    return { desc: 'Výběry', plan: e.plan, reality: e.reality }
                  case 'Sales':
                    return { desc: 'Prodeje', plan: e.plan, reality: e.reality }
                  case 'Reserves':
                    return { desc: 'RES', plan: e.plan, reality: e.reality }
                  default:
                    return e
                }
              }),
              month: coll[0] === undefined ? null : coll[0].thismonth
            }
          })
          .catch(function() {
            return { plans: [], month: null }
          })
      },

      getSubStats: function(id) {
        return getData('target/' + id)
          .then(function(result) {
            return getData('comm?repId=' + id).then(function(commResult) {
              if (result.code != 'OK' || commResult.code != 'OK') {
                return { plans: [], month: null, comm: { weekId: 0 } }
              }

              var coll = result.data.filter(function(row) {
                return (
                  row.desc != 'C1' &&
                  row.desc != 'C2' &&
                  row.desc != 'L1' &&
                  row.desc != 'L2'
                )
              })

              return {
                plans: coll.map(function(e) {
                  switch (e.desc) {
                    case 'Coll':
                      return {
                        desc: 'Výběry',
                        plan: e.plan,
                        reality: e.reality
                      }
                    case 'Sales':
                      return {
                        desc: 'Prodeje',
                        plan: e.plan,
                        reality: e.reality
                      }
                    case 'Reserves':
                      return { desc: 'RES', plan: e.plan, reality: e.reality }
                    default:
                      return e
                  }
                }),
                comm: commResult.data,
                month: coll[0] === undefined ? null : coll[0].thismonth
              }
            })
          })
          .catch(function() {
            return { plans: [], month: null }
          })
      },

      getAgentCommission: function() {
        return getData('comm', 'comm', true)
          .then(function(result) {
            if (result.code != 'OK') {
              return { weekId: 0 }
            }

            return result.data
          })
          .catch(function() {
            return { weekId: 0 }
          })
      },

      getAgentCommissionDetail: function(weekId, chosenView) {
        return UserData.get(
          'comm/' + weekId + (chosenView === null ? '' : '?repId=' + chosenView)
        )
          .then(function(result) {
            if (result.code != 'OK' || result.data === null || result.data === undefined) {
              return null
            }

            return {
              totalAmount: result.data
                .map(function(e) {
                  return e.amount
                })
                .reduce(function(a, b) {
                  return a + b
                }, 0),
              comm: result.data.map(function(e) {
                switch (e.typeName) {
                  case 'DRAWING':
                    return { typeName: 'Výběry', amount: e.amount }
                  case 'ND':
                    return { typeName: 'Noví zákazníci', amount: e.amount }
                  case 'RETURN':
                    return { typeName: 'Vrácené půjčky (ND)', amount: e.amount }
                  default:
                    return e
                }
              })
            }
          })
          .catch(function() {
            return null
          })
      },

      refresh: function() {
        data = {}
        var counter = 0;
        var user;
        var stats;
        var comm;
        var leadss;
        var faultss;

        UserData.refresh()
        return $q(function(resolve) {
          that.getUser().then(function(userData) {
            user = userData;
            counter++;
            if (counter === 5) {
              resolve({
                user: user,
                stats: stats,
                comm: comm,
                leads: leadss,
                faults: faultss, //JSON.parse('[{"Activity":{"AddData":{"Fault":{"@xmlns":"","AgreementNumber":"542599334","Category":"Podpisy smluvní dokumentace","SubCategory":"Smlouva na přepis","Detail":"Nesplňuje","Solution":"Vraťte dokumentaci na pobočku!","Urgency":"1","CisKey":"Podpisy smluvní dokumentace~Smlouva na přepis~Nesplňuje~Vraťte dokumentaci na pobočku!~3226051~0","OriginalOZNum":"1002000250989","OriginalOZName":"Tomcová Hana"}},"AssignedTo":"1002000001479","AssignedToFirstName":"Hana","AssignedToLastName":"Tomcová","Events":[{"AddData":null,"EventDate":"2018-10-26T13:06:26","EventType":"view","HasChangedStatus":false,"Id":2291775,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"České Budějovice BL 11","RecipientFirstName":"František","RecipientLastName":"Šťastný"},{"AddData":null,"EventDate":"2018-10-31T13:49:30","EventType":"view","HasChangedStatus":false,"Id":2292960,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"1002000001479","RecipientFirstName":"Hana","RecipientLastName":"Tomcová"}],"Id":956751,"IdContact":3702893,"IdProcess":369387,"IsScheduled":true,"ScheduledFor":"2018-11-02T00:51:49.007","ScheduledNote":"smlouva na přepis","SystemNote":null,"TimeAction":"2018-11-02T00:51:49.007","TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"},"Contact":{"BankAccountIndicator":0,"BureauConsent":null,"Country":null,"Dob":null,"Email":null,"EmploymentStatus":null,"FirstName":"Regina","Gender":null,"HouseNumber":null,"HousePhoneNumber":null,"Id":3702893,"IdActivity":0,"LastName":"Poklopová","MarketingCommunicationConsent":null,"MobilePhoneNumber":"607266586","Nationality":null,"NegativeBureauConsent":null,"PersonalDataProcessingConsent":null,"Pin":"6751230816","PostalCode":null,"Region":null,"SelfEmpCountry":null,"SelfEmpHouseNumber":"1002000040113","SelfEmpPhoneNumber":null,"SelfEmpPostalCode":null,"SelfEmpRegion":null,"SelfEmpStreetName":null,"SelfEmpTown":null,"StreetName":null,"Title":null,"Town":null,"UirAdrId":null},"Process":{"ActivitiesScheduledNote":null,"AgencyId":null,"ApplicationStage":null,"ChannelCode":null,"CloseReason":null,"ClosedBy":null,"CustomerStatus":null,"FirstResponder":"1002000001479","GDSFlag":"","Id":369387,"IdContact":3702893,"IdLastActivity":956751,"IdNetlemActivityField":12708811,"IdNetlemActivityFieldCorrected":null,"IdNetlemProcess":13616867,"IsClosed":false,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"RequestedAmount":null,"RequestedProductCode":null,"RequestedTerm":null,"ScoringDecision":null,"TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"}}]'),
                that: that,
              })
            }
          }),

            that.getLeads().then(function(leads) {
              leadss = leads;
              counter++;
            if (counter === 5) {
              resolve({
                user: user,
                stats: stats,
                comm: comm,
                leads: leadss,
                faults: faultss, //JSON.parse('[{"Activity":{"AddData":{"Fault":{"@xmlns":"","AgreementNumber":"542599334","Category":"Podpisy smluvní dokumentace","SubCategory":"Smlouva na přepis","Detail":"Nesplňuje","Solution":"Vraťte dokumentaci na pobočku!","Urgency":"1","CisKey":"Podpisy smluvní dokumentace~Smlouva na přepis~Nesplňuje~Vraťte dokumentaci na pobočku!~3226051~0","OriginalOZNum":"1002000250989","OriginalOZName":"Tomcová Hana"}},"AssignedTo":"1002000001479","AssignedToFirstName":"Hana","AssignedToLastName":"Tomcová","Events":[{"AddData":null,"EventDate":"2018-10-26T13:06:26","EventType":"view","HasChangedStatus":false,"Id":2291775,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"České Budějovice BL 11","RecipientFirstName":"František","RecipientLastName":"Šťastný"},{"AddData":null,"EventDate":"2018-10-31T13:49:30","EventType":"view","HasChangedStatus":false,"Id":2292960,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"1002000001479","RecipientFirstName":"Hana","RecipientLastName":"Tomcová"}],"Id":956751,"IdContact":3702893,"IdProcess":369387,"IsScheduled":true,"ScheduledFor":"2018-11-02T00:51:49.007","ScheduledNote":"smlouva na přepis","SystemNote":null,"TimeAction":"2018-11-02T00:51:49.007","TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"},"Contact":{"BankAccountIndicator":0,"BureauConsent":null,"Country":null,"Dob":null,"Email":null,"EmploymentStatus":null,"FirstName":"Regina","Gender":null,"HouseNumber":null,"HousePhoneNumber":null,"Id":3702893,"IdActivity":0,"LastName":"Poklopová","MarketingCommunicationConsent":null,"MobilePhoneNumber":"607266586","Nationality":null,"NegativeBureauConsent":null,"PersonalDataProcessingConsent":null,"Pin":"6751230816","PostalCode":null,"Region":null,"SelfEmpCountry":null,"SelfEmpHouseNumber":"1002000040113","SelfEmpPhoneNumber":null,"SelfEmpPostalCode":null,"SelfEmpRegion":null,"SelfEmpStreetName":null,"SelfEmpTown":null,"StreetName":null,"Title":null,"Town":null,"UirAdrId":null},"Process":{"ActivitiesScheduledNote":null,"AgencyId":null,"ApplicationStage":null,"ChannelCode":null,"CloseReason":null,"ClosedBy":null,"CustomerStatus":null,"FirstResponder":"1002000001479","GDSFlag":"","Id":369387,"IdContact":3702893,"IdLastActivity":956751,"IdNetlemActivityField":12708811,"IdNetlemActivityFieldCorrected":null,"IdNetlemProcess":13616867,"IsClosed":false,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"RequestedAmount":null,"RequestedProductCode":null,"RequestedTerm":null,"ScoringDecision":null,"TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"}}]'),
                that: that,
              })
            }
            }).catch(function(error) { }),
               that.getFaults().then(function(faults) {
                  faultss = faults;
                  counter++;
            if (counter === 5) {
              resolve({
                user: user,
                stats: stats,
                comm: comm,
                leads: leadss,
                faults: faultss, //JSON.parse('[{"Activity":{"AddData":{"Fault":{"@xmlns":"","AgreementNumber":"542599334","Category":"Podpisy smluvní dokumentace","SubCategory":"Smlouva na přepis","Detail":"Nesplňuje","Solution":"Vraťte dokumentaci na pobočku!","Urgency":"1","CisKey":"Podpisy smluvní dokumentace~Smlouva na přepis~Nesplňuje~Vraťte dokumentaci na pobočku!~3226051~0","OriginalOZNum":"1002000250989","OriginalOZName":"Tomcová Hana"}},"AssignedTo":"1002000001479","AssignedToFirstName":"Hana","AssignedToLastName":"Tomcová","Events":[{"AddData":null,"EventDate":"2018-10-26T13:06:26","EventType":"view","HasChangedStatus":false,"Id":2291775,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"České Budějovice BL 11","RecipientFirstName":"František","RecipientLastName":"Šťastný"},{"AddData":null,"EventDate":"2018-10-31T13:49:30","EventType":"view","HasChangedStatus":false,"Id":2292960,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"1002000001479","RecipientFirstName":"Hana","RecipientLastName":"Tomcová"}],"Id":956751,"IdContact":3702893,"IdProcess":369387,"IsScheduled":true,"ScheduledFor":"2018-11-02T00:51:49.007","ScheduledNote":"smlouva na přepis","SystemNote":null,"TimeAction":"2018-11-02T00:51:49.007","TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"},"Contact":{"BankAccountIndicator":0,"BureauConsent":null,"Country":null,"Dob":null,"Email":null,"EmploymentStatus":null,"FirstName":"Regina","Gender":null,"HouseNumber":null,"HousePhoneNumber":null,"Id":3702893,"IdActivity":0,"LastName":"Poklopová","MarketingCommunicationConsent":null,"MobilePhoneNumber":"607266586","Nationality":null,"NegativeBureauConsent":null,"PersonalDataProcessingConsent":null,"Pin":"6751230816","PostalCode":null,"Region":null,"SelfEmpCountry":null,"SelfEmpHouseNumber":"1002000040113","SelfEmpPhoneNumber":null,"SelfEmpPostalCode":null,"SelfEmpRegion":null,"SelfEmpStreetName":null,"SelfEmpTown":null,"StreetName":null,"Title":null,"Town":null,"UirAdrId":null},"Process":{"ActivitiesScheduledNote":null,"AgencyId":null,"ApplicationStage":null,"ChannelCode":null,"CloseReason":null,"ClosedBy":null,"CustomerStatus":null,"FirstResponder":"1002000001479","GDSFlag":"","Id":369387,"IdContact":3702893,"IdLastActivity":956751,"IdNetlemActivityField":12708811,"IdNetlemActivityFieldCorrected":null,"IdNetlemProcess":13616867,"IsClosed":false,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"RequestedAmount":null,"RequestedProductCode":null,"RequestedTerm":null,"ScoringDecision":null,"TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"}}]'),
                that: that,
              })
            }
               }),
                that.getStats().then(function(statsData) {
                  stats = statsData;
                  counter++;
            if (counter === 5) {
              resolve({
                user: user,
                stats: stats,
                comm: comm,
                leads: leadss,
                faults: faultss, //JSON.parse('[{"Activity":{"AddData":{"Fault":{"@xmlns":"","AgreementNumber":"542599334","Category":"Podpisy smluvní dokumentace","SubCategory":"Smlouva na přepis","Detail":"Nesplňuje","Solution":"Vraťte dokumentaci na pobočku!","Urgency":"1","CisKey":"Podpisy smluvní dokumentace~Smlouva na přepis~Nesplňuje~Vraťte dokumentaci na pobočku!~3226051~0","OriginalOZNum":"1002000250989","OriginalOZName":"Tomcová Hana"}},"AssignedTo":"1002000001479","AssignedToFirstName":"Hana","AssignedToLastName":"Tomcová","Events":[{"AddData":null,"EventDate":"2018-10-26T13:06:26","EventType":"view","HasChangedStatus":false,"Id":2291775,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"České Budějovice BL 11","RecipientFirstName":"František","RecipientLastName":"Šťastný"},{"AddData":null,"EventDate":"2018-10-31T13:49:30","EventType":"view","HasChangedStatus":false,"Id":2292960,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"1002000001479","RecipientFirstName":"Hana","RecipientLastName":"Tomcová"}],"Id":956751,"IdContact":3702893,"IdProcess":369387,"IsScheduled":true,"ScheduledFor":"2018-11-02T00:51:49.007","ScheduledNote":"smlouva na přepis","SystemNote":null,"TimeAction":"2018-11-02T00:51:49.007","TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"},"Contact":{"BankAccountIndicator":0,"BureauConsent":null,"Country":null,"Dob":null,"Email":null,"EmploymentStatus":null,"FirstName":"Regina","Gender":null,"HouseNumber":null,"HousePhoneNumber":null,"Id":3702893,"IdActivity":0,"LastName":"Poklopová","MarketingCommunicationConsent":null,"MobilePhoneNumber":"607266586","Nationality":null,"NegativeBureauConsent":null,"PersonalDataProcessingConsent":null,"Pin":"6751230816","PostalCode":null,"Region":null,"SelfEmpCountry":null,"SelfEmpHouseNumber":"1002000040113","SelfEmpPhoneNumber":null,"SelfEmpPostalCode":null,"SelfEmpRegion":null,"SelfEmpStreetName":null,"SelfEmpTown":null,"StreetName":null,"Title":null,"Town":null,"UirAdrId":null},"Process":{"ActivitiesScheduledNote":null,"AgencyId":null,"ApplicationStage":null,"ChannelCode":null,"CloseReason":null,"ClosedBy":null,"CustomerStatus":null,"FirstResponder":"1002000001479","GDSFlag":"","Id":369387,"IdContact":3702893,"IdLastActivity":956751,"IdNetlemActivityField":12708811,"IdNetlemActivityFieldCorrected":null,"IdNetlemProcess":13616867,"IsClosed":false,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"RequestedAmount":null,"RequestedProductCode":null,"RequestedTerm":null,"ScoringDecision":null,"TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"}}]'),
                that: that,
              })
            }
                }),
                  that.getAgentCommission().then(function(commData) {
                    comm = commData;
                    counter++;
            if (counter === 5) {
              resolve({
                user: user,
                stats: stats,
                comm: comm,
                leads: leadss,
                faults: faultss, //JSON.parse('[{"Activity":{"AddData":{"Fault":{"@xmlns":"","AgreementNumber":"542599334","Category":"Podpisy smluvní dokumentace","SubCategory":"Smlouva na přepis","Detail":"Nesplňuje","Solution":"Vraťte dokumentaci na pobočku!","Urgency":"1","CisKey":"Podpisy smluvní dokumentace~Smlouva na přepis~Nesplňuje~Vraťte dokumentaci na pobočku!~3226051~0","OriginalOZNum":"1002000250989","OriginalOZName":"Tomcová Hana"}},"AssignedTo":"1002000001479","AssignedToFirstName":"Hana","AssignedToLastName":"Tomcová","Events":[{"AddData":null,"EventDate":"2018-10-26T13:06:26","EventType":"view","HasChangedStatus":false,"Id":2291775,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"České Budějovice BL 11","RecipientFirstName":"František","RecipientLastName":"Šťastný"},{"AddData":null,"EventDate":"2018-10-31T13:49:30","EventType":"view","HasChangedStatus":false,"Id":2292960,"IdActivity":956751,"IsActionable":false,"IsLogOnly":true,"IsStandard":true,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"Note":"","Recipient":"1002000001479","RecipientFirstName":"Hana","RecipientLastName":"Tomcová"}],"Id":956751,"IdContact":3702893,"IdProcess":369387,"IsScheduled":true,"ScheduledFor":"2018-11-02T00:51:49.007","ScheduledNote":"smlouva na přepis","SystemNote":null,"TimeAction":"2018-11-02T00:51:49.007","TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"},"Contact":{"BankAccountIndicator":0,"BureauConsent":null,"Country":null,"Dob":null,"Email":null,"EmploymentStatus":null,"FirstName":"Regina","Gender":null,"HouseNumber":null,"HousePhoneNumber":null,"Id":3702893,"IdActivity":0,"LastName":"Poklopová","MarketingCommunicationConsent":null,"MobilePhoneNumber":"607266586","Nationality":null,"NegativeBureauConsent":null,"PersonalDataProcessingConsent":null,"Pin":"6751230816","PostalCode":null,"Region":null,"SelfEmpCountry":null,"SelfEmpHouseNumber":"1002000040113","SelfEmpPhoneNumber":null,"SelfEmpPostalCode":null,"SelfEmpRegion":null,"SelfEmpStreetName":null,"SelfEmpTown":null,"StreetName":null,"Title":null,"Town":null,"UirAdrId":null},"Process":{"ActivitiesScheduledNote":null,"AgencyId":null,"ApplicationStage":null,"ChannelCode":null,"CloseReason":null,"ClosedBy":null,"CustomerStatus":null,"FirstResponder":"1002000001479","GDSFlag":"","Id":369387,"IdContact":3702893,"IdLastActivity":956751,"IdNetlemActivityField":12708811,"IdNetlemActivityFieldCorrected":null,"IdNetlemProcess":13616867,"IsClosed":false,"LeadStatusCode1":null,"LeadStatusCode2":null,"LeadStatusId":null,"LeadStatusName1":null,"LeadStatusName2":null,"RequestedAmount":null,"RequestedProductCode":null,"RequestedTerm":null,"ScoringDecision":null,"TimeEnd":null,"TimeStart":"2018-10-19T00:51:49.007"}}]'),
                that: that,
              })
            }
                  })

                })
              }
      ,

      softRefresh: function() {
        return UserData.get('api')
          .then(function(result) {
            if (result.code == 'OK') {
              return that.refresh()
            } else {
              return $q(function(resolve, reject) {
                reject()
              })
            }
          })
          .catch(function() {
            return $q(function(resolve, reject) {
              reject()
            })
          })
      }
    }

    return that
  })
